# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: workspace_meta.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import object_identifier_pb2 as object__identifier__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14workspace_meta.proto\x12\x08\x63z.proto\x1a\x17object_identifier.proto\"t\n\tWorkspace\x12\x10\n\x08location\x18\x01 \x01(\t\x12\x1a\n\x12optional_locations\x18\x02 \x03(\t\x12+\n\x05share\x18\n \x01(\x0b\x32\x1a.cz.proto.ObjectIdentifierH\x00\x42\x0c\n\nconnectionb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'workspace_meta_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _WORKSPACE._serialized_start=59
  _WORKSPACE._serialized_end=175
# @@protoc_insertion_point(module_scope)
