# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: bucket_info.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x62ucket_info.proto\x12\x08\x63z.proto\"2\n\nBucketInfo\x12\x11\n\tbucket_id\x18\x01 \x01(\r\x12\x11\n\tblock_ids\x18\x02 \x03(\rB\n\n\x08\x63z.protob\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bucket_info_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\010cz.proto'
  _BUCKETINFO._serialized_start=31
  _BUCKETINFO._serialized_end=81
# @@protoc_insertion_point(module_scope)
